﻿#ifndef engine_audio_mixing_manager_h
#define engine_audio_mixing_manager_h

#include "engine_utils.h"
#include "engine_types.h"
#include <stdio.h>
#include <stdint.h>
#include <string>

namespace ding {
namespace rtc {

/** \defgroup CPP_DingRtcEngineAudioMixingManager 伴奏混音相关功能接口
 *  DingRtc 伴奏混音相关功能接口
 */

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.4
 * @brief 伴奏混音状态。
 */        
typedef enum {
    /** 初始状态。 */
    kAudioMixingNone = 0,
    /** 开始混音。 */
    kAudioMixingStarted = 100,
    /** 停止混音。 */
    kAudioMixingStopped = 101,
    /** 混音暂停。 */
    kAudioMixingPaused = 102,
    /** 混音恢复。 */
    kAudioMixingResumed = 103,
    /** 混音完毕。 */
    kAudioMixingEnded = 104,
    /** 正在缓冲。 */
    kAudioMixingBuffering = 105,
    /** 缓冲结束。 */
    kAudioMixingBufferingEnd = 106,
    /** 混音失败。 */
    kAudioMixingFailed = 107,
} RtcEngineAudioMixingStatus;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.4
 * @brief 伴奏混音错误码。
 */
typedef enum {
    /** 无错误。 */
    kAudioMixingNoError = 0,
    /** 打开文件失败。 */
    kAudioMixingOpenFailed = -100,
    /** 解码失败。 */
    kAudioMixingDecodeFailed = -101,
} RtcEngineAudioMixingErrorCode;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.4
 * @brief 伴奏混音状态信息。
 */
struct RtcEngineAudioMixingStatusConfig {
    /** 伴奏混音状态。 */
    RtcEngineAudioMixingStatus status;
    /** 伴奏混音错误信息。 */
    RtcEngineAudioMixingErrorCode errorCode;
    /** 伴奏文件名称。 */
    std::string fileName;
    /** 伴奏文件id。 */
    int id;
    /** 伴奏时长。 */
    long durationMs;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.4
 * @brief 伴奏混音配置信息。
 */
struct RtcEngineAudioMixingConfig {
    /** 是否允许伴奏推流到远端，true：允许推流，false：不允许推流。*/
    bool enablePublish;
    /** 是否允许伴奏本地播放，true：允许播放，false：不允许播放。*/
    bool enablePlayout;
    /** 伴奏推流到远端音量，取值范围[0, 100]。*/
    int publishVolume;
    /** 伴奏本地播放音量，取值范围[0, 100]。*/
    int playoutVolume;
    /** 循环次数，大于0有效。*/
    int cycles;
    /** 开始播放位置，单位ms。*/
    long startPosMs;

    RtcEngineAudioMixingConfig()
    : enablePublish(false)
    , enablePlayout(true)
    , publishVolume(100)
    , playoutVolume(100)
    , cycles(1)
    , startPosMs(0) {
    }
};

/**
 * @brief 伴奏混音相关功能。
*/
class RtcEngineAudioMixingManager {
public:
    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 创建伴奏混音实例。
     * @param id 伴奏文件id，用户分配。
     * @param filePath 伴奏文件路径。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int CreateAudioMixing(int id, const ding::rtc::String& filePath) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 销毁伴奏混音实例。
     * @param id 伴奏文件id。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int DestroyAudioMixing(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 开始伴奏混音。
     * @param id 伴奏文件id。
     * @param config 伴奏文件配置，参考{@link RtcEngineAudioMixingConfig}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 可通过{@link RtcEngineEventListener#OnAudioMixingStateChanged}监听播放状态。
     */
    virtual int StartAudioMixing(int id, RtcEngineAudioMixingConfig& config) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 停止伴奏混音。
     * @param id 伴奏文件id。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int StopAudioMixing(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 暂停伴奏混音。
     * @param id 伴奏文件id。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int PauseAudioMixing(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 暂停后恢复伴奏混音。
     * @param id 伴奏文件id。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int ResumeAudioMixing(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 获取伴奏时长。
     * @param id 伴奏文件id。
     * @return 伴奏时长，单位ms。返回值为-1时，说明获取时长失败。
     */
    virtual long GetAudioMixingDuration(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 获取当前伴奏播放时长。
     * @param id 伴奏文件id。
     * @return 当前伴奏播放时长，单位ms。返回值为-1时，说明获取时长失败。
     */
    virtual long GetAudioMixingCurrentPosition(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 设置伴奏播放位置。
     * @param id 伴奏文件id。
     * @param position 设置伴奏播放位置，单位ms。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetAudioMixingPosition(int id, long position) = 0;
    
    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 设置伴奏音量。
     * @param id 伴奏文件id。
     * @param volume 伴奏音量，取值[0, 100]。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetAudioMixingVolume(int id, int volume) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 获取伴奏音量。
     * @param id 伴奏文件id。
     * @return 伴奏音量， < 0 代表获取失败。
     */
    virtual int GetAudioMixingVolume(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 设置伴奏推流音量。
     * @param id 伴奏文件id。
     * @param volume 伴奏推流音量，取值[0, 100]。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetAudioMixingPublishVolume(int id, int volume) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 获取伴奏推流音量。
     * @param id 伴奏文件id。
     * @return 伴奏推流音量， < 0 代表获取失败。
     */
    virtual int GetAudioMixingPublishVolume(int id) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 设置伴奏播放音量。
     * @param id 伴奏文件id。
     * @param volume 伴奏播放音量，取值[0, 100]。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetAudioMixingPlayoutVolume(int id, int volume) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioMixingManager
     * @since 3.4
     * @brief 获取伴奏播放音量。
     * @param id 伴奏文件id。
     * @return 伴奏播放音量， < 0 代表获取失败。
     */
    virtual int GetAudioMixingPlayoutVolume(int id) = 0;

protected:
    virtual ~RtcEngineAudioMixingManager() {}
};

} // namespace rtc
} // namespace ding


#endif /* engine_audio_mixing_manager_h */
